%define realname lua
%define sover 5
%define weight 20

Name: %realname%sover
Version: 5.0
Release: alt7.1

Summary: Lua is a powerful, light-weight programming language
License: MIT
Group: Development/Other

Url: http://www.lua.org
# http://www.lua.org/ftp/
Source: %realname-%version.tar.bz2

# under alternatives control
%set_compress_method gzip
Conflicts: lua4 < 4.0.1-alt7

# known bugs in Lua: http://www.lua.org/bugs.html
# 1. lua_closethread exists only in the manual
# Patch: The manual is wrong: threads are subject to garbage collection.
# 2. attempt to resume a running coroutine crashes Lua
Patch2: %realname-5.0-lua-ldo-coroutine.patch
# 3. file:close cannot be called without a file. (results in seg. fault)
Patch3: %realname-5.0-lua-liolib-file-close.patch
# 4. C functions may have stacks larger than current top
Patch4: %realname-5.0-lua-lgc-stack.patch
# 5. `pc' address is invalidated when a coroutine is suspended
Patch5: %realname-5.0-lua-lvm-coroutine.patch
# 6. userdata to be collected still counts into new GC threshold, increasing memory consumption
Patch6: %realname-5.0-lua-lgc-memleak.patch
# 7. IBM AS400 (OS400) has sizeof(void *)==16, and a `%p' may generate up to 60 characters in a `printf'. That causes a buffer overflow in `tostring'.
Patch7: %realname-5.0-lua-printf-overflow.patch
# 8. syntax `local function' does not increment stack size
Patch8: %realname-5.0-lua-lparser-localfunc.patch
# 9. count hook may be called without being set
Patch9: %realname-5.0-lua-lvm-count-hook.patch

# debian
Patch99: %realname-5.0-deb-paths.patch

PreReq: alternatives >= 0.0.6
Requires: lib%name = %version-%release
Provides:  %realname = %version
Obsoletes: %realname

# Automatically added by buildreq on Sun Aug 24 2003
BuildRequires: libalternatives-devel libncurses-devel libreadline-devel

%package -n lib%name
Summary: Lua programming language, shared libraries
Group: System/Libraries

%package -n lib%name-devel
Summary: Lua programming language, header files
Group: Development/Other
Requires: lib%name = %version-%release
Provides: %realname-devel = %version, lib%realname-devel = %version
Obsoletes: %realname-devel, lib%realname-devel
Conflicts: liblua4-devel

%package -n lib%name-devel-static
Summary: Lua programming language, static libraries
Group: Development/Other
Requires: lib%name-devel = %version-%release
Provides: %realname-devel-static = %version, lib%realname-devel-static = %version
Obsoletes: %realname-devel-static, lib%realname-devel-static
Conflicts: liblua4-devel-static

%description
Lua is a powerful, light-weight programming language designed for extending
applications.  Lua is also frequently used as a general-purpose, stand-alone
language.

This package contains lua compiler and interpreter, as well as code samples
and reference manual of the language.

%description -n lib%name
Lua is a powerful, light-weight programming language designed for extending
applications.  Lua is also frequently used as a general-purpose, stand-alone
language.

This package contains lua runtime shared libraries.

%description -n lib%name-devel
Lua is a powerful, light-weight programming language designed for extending
applications.  Lua is also frequently used as a general-purpose, stand-alone
language.

This package contains header files needed to embed Lua in C/C++ programs.

%description -n lib%name-devel-static
Lua is a powerful, light-weight programming language designed for extending
applications.  Lua is also frequently used as a general-purpose, stand-alone
language.

This package contains static libraries one may need to embed lua statically.

%prep
%setup -q -n %realname-%version
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch99 -p1

%build
%def_enable Werror

: custom build
DEST=$PWD/ready
%__mkdir_p $DEST

# tweak stdio.h with -D_GNU_SOURCE to get popen(3) and pclose(3) prototypes
CFLAGS="-I$PWD/include -I$PWD/src -I$PWD/etc -D_GNU_SOURCE=1
	-DUSE_POPEN=1 -DUSE_DLOPEN=1 -DUSE_FASTROUND=1 -DUSE_READLINE=1
	-DLUA_USER_H=\"luser_number.h\" -DLUA_USERCONFIG=\"$PWD/etc/saconfig.c\""

: static libraries
cd ./src
gcc $CFLAGS %optflags -c *.c
ar rcu $DEST/liblua.a *.o
ranlib $DEST/liblua.a

cd ./lib
gcc $CFLAGS %optflags -c *.c
ar rcu $DEST/liblualib.a *.o
ranlib $DEST/liblualib.a

: dynamic libraries
cd ..
gcc $CFLAGS %optflags %optflags_shared -c *.c
ld -o $DEST/liblua.so.%sover -soname=liblua.so.%sover -shared -O1 -lc *.o
ln -s liblua.so.%sover $DEST/liblua.so

cd ./lib
gcc $CFLAGS %optflags %optflags_shared -c *.c
ld -o $DEST/liblualib.so.%sover -soname=liblualib.so.%sover -shared -O1 -lc -lm -ldl *.o
ln -s liblualib.so.%sover $DEST/liblualib.so

: executables
cd ../lua
gcc $CFLAGS %optflags *.c -o $DEST/lua -L$DEST \
	-llua -llualib -lreadline -lcurses
cd ../luac 
gcc $CFLAGS %optflags -DLUA_OPNAMES *.c ../lopcodes.c \
	-o $DEST/luac -L$DEST -llua -llualib

%install
%__mkdir_p %buildroot{%_bindir,%_libdir,%_includedir,%_man1dir,%_altdir,%_datadir/%name/{etc,test}}
%__cp -a ready/*.a ready/*.so* %buildroot%_libdir/
%__cp -a include/*.h %buildroot%_includedir/
%__install -p -m644 test/*.lua test/README \
	%buildroot%_datadir/%name/test/
%__install -p -m644 etc/*.* etc/README \
	%buildroot%_datadir/%name/etc/
%__ln_s etc/compat.lua \
	%buildroot%_datadir/%name/

# alternatives
%__cp -a ready/lua %buildroot%_bindir/lua%sover
%__cp -a ready/luac %buildroot%_bindir/luac%sover
%__cp -a doc/lua.1 %buildroot%_man1dir/lua%sover.1
%__cp -a doc/luac.1 %buildroot%_man1dir/luac%sover.1

# docs
%__mkdir -p %buildroot%_docdir/%name-%version
%__cp -a COPYRIGHT HISTORY README* doc/*.html doc/*.gif %buildroot%_docdir/%name-%version

%__cat <<EOF >%buildroot%_altdir/%name
<group name="candidate">
	<option name="link">%_bindir/lua</option>
	<option name="real">%_bindir/lua%sover</option>
	<option name="weight" type="number">%weight</option>
	<group name="slave">
		<option name="link">%_bindir/luac.1.gz</option>
		<option name="real">%_bindir/luac%sover.1.gz</option>
	</group>
	<group name="slave">
		<option name="link">%_man1dir/%realname.1.gz</option>
		<option name="real">%_man1dir/%realname%sover.1.gz</option>
	</group>
	<group name="slave">
		<option name="link">%_man1dir/luac.1.gz</option>
		<option name="real">%_man1dir/luac%sover.1.gz</option>
	</group>
</group>
EOF

%post
%register_alternatives %name

%preun
%unregister_alternatives %name

%post -n lib%name -p %post_ldconfig
%postun -n lib%name -p %postun_ldconfig

%files
%_altdir/*
%_bindir/lua*
%_man1dir/*
%_datadir/%name
%dir %_docdir/%name-%version
%_docdir/%name-%version/*.html
%_docdir/%name-%version/*.gif

%files -n lib%name
%_libdir/*.so.*
%dir %_docdir/%name-%version
%_docdir/%name-%version/COPYRIGHT
%_docdir/%name-%version/HISTORY
%_docdir/%name-%version/README*

%files -n lib%name-devel
%_libdir/*.so
%_includedir/*.h

%files -n lib%name-devel-static
%_libdir/*.a

%changelog
* Mon Feb 02 2004 Alexey Tourbin <at@altlinux.ru> 5.0-alt7.1
- fixed %_docdir/%name-%version ownership

* Fri Jan 30 2004 Alexey Tourbin <at@altlinux.ru> 5.0-alt7
- fixed _customdocdir misusage
- Werror mode enabled

* Tue Nov 18 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt6
- 5 more official patches applied

* Sun Aug 24 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt5
- moved lua4 (weight=10) and lua5 (weight=20) packages under alternatives control
- in sync with debian (deb-paths.patch):
  + Added support for -C to load compatibility library on startup
  + Added an initial LUA_PATH to support /usr/share/lua5 etc.
  + Added a -P switch to suppress the LUA_PATH code if need-be
  + Updated the manpage lua.1 for the -C and -P switches

* Fri Jun 20 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt4
- packages renamed: s/lua/lua5/
- lua4 will be kept; thus Provides/Obsoletes reconsidered, Conflicts added
- two patches from http://www.lua.org/bugs.html
- Sisyphus release

* Wed May 07 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt3
- added Provides and Obsoletes for smooth upgrade

* Mon May 05 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt2
- desert Makefile/config/patches; custom build with direct commands
- features enabled: dlopen (loadlib), popen, fastround, libreadline
- new package layout: lua, liblua, liblua-devel, liblua-devel-static

* Mon Apr 28 2003 Alexey Tourbin <at@altlinux.ru> 5.0-alt1
- 5.0; API changes; soname changed; license: MIT
- loadlib now implemented, so drop that stuff borrowed from PLD

* Sun Jan 19 2003 Alexey Tourbin <at@altlinux.ru> 4.0.1-alt6
- minor enhancements

* Thu Dec 26 2002 Alexey Tourbin <at@altlinux.ru> 4.0.1-alt5
- corrected s/-lc/-lc -lm -ldl/ for liblualib.so in alt-soname patch
- shared optimization flags

* Tue Dec 24 2002 Alexey Tourbin <at@altlinux.ru> 4.0.1-alt4
- error in previous build: *.so files are binary copy of *.so.* files,
  not symlinks; fixed
- really shared build (s/ld -shared/ld -shared -lc/ in alt-soname patch)
- post and postun sections added (ldconfig)

* Fri Dec 20 2002 Alexey Tourbin <at@altlinux.ru> 4.0.1-alt3
- shared build; alt-soname patch; lua-devel-static subpackage
- reference manual and code samples included in lua-devel package
- additional features from XTG Systems (loadlib files) and
  PLD Team (pld-loadlib-require patch)
- buildreq applied (none)

* Mon Oct 28 2002 Konstantin Volckov <goldhead@altlinux.ru> 4.0.1-alt2
- Rebuilt in new environment

* Wed Aug 28 2002 Konstantin Volckov <goldhead@altlinux.ru> 4.0.1-alt1
- First build for Sisyphus
